// Copyright (C) 2011-2012 R M Yorston
// Licence: GPLv2+

const Lang = imports.lang;
const Mainloop = imports.mainloop;

const Clutter = imports.gi.Clutter;
const EndSessionDialog = imports.ui.endSessionDialog;
const Main = imports.ui.main;
const UserMenu = imports.ui.userMenu;

const _f = imports.gettext.domain('frippery-shut-down-menu').gettext;

const shutdownDialogContent = {
    subject: C_("title", "Power Off"),
    inhibitedDescription: _("Click Power Off to quit these applications and power off the system."),
    uninhibitedDescription: function(seconds) {
        return ngettext("The system will power off automatically in %d second.",
                        "The system will power off automatically in %d seconds.",
                        seconds).format(seconds);
    },
    endDescription: _("Powering off the system."),
    secondaryButtons: [{ signal: 'ConfirmedSuspend',
                       label:  _("Suspend") },
                     { signal: 'ConfirmedHibernate',
                       label:  _f("Hibernate") },
                     { signal: 'ConfirmedReboot',
                       label:  C_("button", "Restart") }],
    confirmButtons: [{ signal: 'ConfirmedShutdown',
                       label:  C_("button", "Power Off") }],
    iconName: 'system-shutdown-symbolic',
    iconStyleClass: 'end-session-dialog-shutdown-icon'
};

let usermenu = null;
let myUpdateSuspendOrPowerOff, origUpdateSuspendOrPowerOff;
let myUpdateButtons, origUpdateButtons;
let originalDialogContent;

function init(extensionMeta) {
    let localePath = extensionMeta.path + '/locale';
    imports.gettext.bindtextdomain('frippery-shut-down-menu', localePath);

    usermenu = Main.panel.statusArea['userMenu'];

    if ( !usermenu ) {
        return;
    }

    originalDialogContent = EndSessionDialog.DialogContent[1];

    origUpdateSuspendOrPowerOff = usermenu._updateSuspendOrPowerOff;
    myUpdateSuspendOrPowerOff = function() {
        this._haveSuspend = this._upClient.get_can_suspend();

        if (!this._suspendOrPowerOffItem)
            return;

        this._suspendOrPowerOffItem.actor.visible = this._haveShutdown || this._haveSuspend;

        if ( !this._haveShutdown && this._haveSuspend ) {
            this._suspendOrPowerOffItem.updateText(_f("Suspend"), null);
        }
        else {
            this._suspendOrPowerOffItem.updateText(_f("Shut Down..."), null);
        }
    };

    EndSessionDialog.EndSessionDialog.prototype._onVisible = function() {
        this._group.disconnect(this._visibilityNotifyId);
        if ( this._group.visible ) {
            return;
        }

        if (this._screenSaverSettings.get_boolean(UserMenu.LOCK_ENABLED_KEY)) {
            let tmpId = Main.screenShield.connect('lock-screen-shown',
                Lang.bind(this, function() {
                    Main.screenShield.disconnect(tmpId);
                    Mainloop.timeout_add(2000, Lang.bind(this, function() {
                        this._shutdownAction();
                        return false;
                    }));
                }));

            Main.screenShield.lock(true);
        }
        else {
            this._shutdownAction();
        }
    };

    EndSessionDialog.EndSessionDialog.prototype._doHibernate = function() {
        this._upClient.hibernate_sync(null);
    };

    EndSessionDialog.EndSessionDialog.prototype._onHibernate = function() {
        this._stopTimer();
        this._dbusImpl.emit_signal('Canceled', null);

        this._shutdownAction = Lang.bind(this, this._doHibernate);
        this._visibilityNotifyId = this._group.connect('notify::visible',
            Lang.bind(this, this._onVisible));

        this.close(global.get_current_time());
    };

    EndSessionDialog.EndSessionDialog.prototype._doSuspend = function() {
        this._upClient.suspend_sync(null);
    };

    EndSessionDialog.EndSessionDialog.prototype._onSuspend = function() {
        this._stopTimer();
        this._dbusImpl.emit_signal('Canceled', null);

        this._shutdownAction = Lang.bind(this, this._doSuspend);
        this._visibilityNotifyId = this._group.connect('notify::visible',
            Lang.bind(this, this._onVisible));

        this.close(global.get_current_time());
    };

    origUpdateButtons =
        EndSessionDialog.EndSessionDialog.prototype._updateButtons;

    myUpdateButtons = function() {
        let dialogContent = EndSessionDialog.DialogContent[this._type];
        let buttons = [];
        let usermenu = Main.panel.statusArea['userMenu'];
        this._upClient = usermenu._upClient;
        this._screenSaverSettings = usermenu._screenSaverSettings;

        if ( dialogContent.secondaryButtons ) {
            for (let i = 0; i < dialogContent.secondaryButtons.length; i++) {
                let signal = dialogContent.secondaryButtons[i].signal;
                let label = dialogContent.secondaryButtons[i].label;

                if ( signal == 'ConfirmedHibernate' ) {
                    if ( this._upClient && this._upClient.get_can_hibernate() ) {
                        buttons.push({ action: Lang.bind(this, this._onHibernate),
                                       label: label });
                    }
                }
                else if ( signal == 'ConfirmedSuspend' ) {
                    if ( this._upClient && this._upClient.get_can_suspend() ) {
                        buttons.push({ action: Lang.bind(this, this._onSuspend),
                                       label: label });
                    }
                }
                else {
                    buttons.push({ action: Lang.bind(this, function() {
                                           this._confirm(signal);
                                       }),
                                   label: label });
                }
            }
        }

        buttons.push({ action: Lang.bind(this, this.cancel),
                         label:  _("Cancel"),
                         key:    Clutter.Escape });

        for (let i = 0; i < dialogContent.confirmButtons.length; i++) {
            let signal = dialogContent.confirmButtons[i].signal;
            let label = dialogContent.confirmButtons[i].label;
            buttons.push({ action: Lang.bind(this, function() {
                                       this._confirm(signal);
                                   }),
                           label: label });
        }

        this.setButtons(buttons);
    };
}

function enable() {
    if ( usermenu ) {
        usermenu._updateSuspendOrPowerOff = myUpdateSuspendOrPowerOff;
        EndSessionDialog.EndSessionDialog.prototype._updateButtons =
            myUpdateButtons;
        EndSessionDialog.DialogContent[1] = shutdownDialogContent;
        usermenu._updateSuspendOrPowerOff();
    }
}

function disable() {
    if ( usermenu ) {
        usermenu._updateSuspendOrPowerOff = origUpdateSuspendOrPowerOff;
        EndSessionDialog.EndSessionDialog.prototype._updateButtons =
            origUpdateButtons;
        EndSessionDialog.DialogContent[1] = originalDialogContent;
        usermenu._updateSuspendOrPowerOff();
    }
}
